SystemOrganization addCategory: #'Seaside-RenderLoop'!

WAObject subclass: #WARenderLoop
	instanceVariableNames: 'root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WARenderLoop methodsFor: 'processing' stamp: 'lr 3/29/2008 14:20'!
call: aComponent 
	^ WAContinuation currentDo: [ :cc | 
		aComponent onAnswer: cc.
		self root: aComponent.
		self run ]! !

!WARenderLoop methodsFor: 'accessing-properties' stamp: 'lr 8/17/2007 09:56'!
renderContinuationClass
	^ self application preferenceAt: #renderContinuationClass! !

!WARenderLoop methodsFor: 'accessing' stamp: 'lr 4/20/2007 15:12'!
root
	^ root! !

!WARenderLoop methodsFor: 'accessing' stamp: 'jf 9/13/2008 09:53'!
root: aComponent
	root := aComponent! !

!WARenderLoop methodsFor: 'processing' stamp: 'lr 8/17/2007 09:54'!
run
	(self renderContinuationClass root: self root) run! !

WAObject subclass: #WASessionContinuation
	instanceVariableNames: 'root states'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WASessionContinuation commentStamp: 'lr 6/2/2008 09:19' prior: 0!
I represent a continuation as part of the flow of pages within a session. I am not a real continuation (as compared to those used in #call: and #answer:), I only represent a specific point in the session. I reference the root component and a memory snapshot of backtracked objects.!

WASessionContinuation subclass: #WARedirectContinuation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WARedirectContinuation commentStamp: 'lr 6/2/2008 09:22' prior: 0!
I redirect to a WARenderContinuation. I am necessary to avoid the execution of anchor and form callbacks, when the back-button is used.!

!WARedirectContinuation methodsFor: 'processing' stamp: 'lr 8/17/2007 09:54'!
handleRequest: aRequest
	self newRenderContinuation run! !

!WARedirectContinuation methodsFor: 'processing' stamp: 'jf 9/22/2008 21:06'!
run
	self respond: [ :response |
		response redirectTo: self url ]! !

WASessionContinuation subclass: #WARenderContinuation
	instanceVariableNames: 'renderContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WARenderContinuation commentStamp: 'lr 6/2/2008 09:21' prior: 0!
I am used to trigger the rendering of the component-tree. After processing the callbacks of the page I rendered, I am followed by a WARedirectContinuation.!

!WARenderContinuation methodsFor: 'processing' stamp: 'lr 4/9/2008 10:52'!
handleRequest: aRequest 
	aRequest isXmlHttpRequest ifTrue: [ self requestContext onRespond: [ :r | states snapshot ] ].
	self withNotificationHandler: [ self processCallbacks: aRequest ].
	((self shouldRedirect: aRequest) 
		ifTrue: [ self newRedirectContinuation ]
		ifFalse: [ self newRenderContinuation ]) run! !

!WARenderContinuation methodsFor: 'creational' stamp: 'jf 9/28/2008 17:29'!
newContext
	^ WARenderingContext new
		actionUrl: self url;
		yourself! !

!WARenderContinuation methodsFor: 'creational' stamp: 'lr 3/29/2008 22:20'!
newHtmlRoot
	| htmlRoot |
	htmlRoot := WAHtmlRoot context: renderContext.
	htmlRoot bodyAttributes at: 'onload' append: 'onLoad()' separator: ';'.
	self updateRoot: htmlRoot.
	^ htmlRoot! !

!WARenderContinuation methodsFor: 'private' stamp: 'jf 9/28/2008 21:42'!
processCallbacks: aRequest
	renderContext callbacks processWithRequest: aRequest! !

!WARenderContinuation methodsFor: 'private' stamp: 'jf 7/16/2008 09:41'!
processRendering: aResponse 
	| document htmlRoot |
	document := self application newDocumentOn: aResponse stream.
	renderContext document: document.
	htmlRoot := self newHtmlRoot.
	document open: htmlRoot.
	self root decorationChainDo: [ :each | each renderWithContext: renderContext ].
	self writeOnLoadOn: document.
	document close: htmlRoot.
	renderContext release! !

!WARenderContinuation methodsFor: 'actions' stamp: 'lr 3/29/2008 22:15'!
render
	self session clearJumpTo.
	renderContext := self newContext.
	self respond: [ :response | self processRendering: response ]! !

!WARenderContinuation methodsFor: 'processing' stamp: 'lr 4/20/2007 19:41'!
run
	self withNotificationHandler: [ self render ]! !

!WARenderContinuation methodsFor: 'testing' stamp: 'lr 4/16/2007 21:06'!
shouldRedirect: aRequest
	^ aRequest isGet not or: [ (aRequest fields includesKey: '_n') not ]! !

!WARenderContinuation methodsFor: 'private' stamp: 'jf 9/14/2008 15:15'!
withNotificationHandler: aBlock
	^ aBlock on: WARenderNotification do: [ :n | ]! !

!WARenderContinuation methodsFor: 'private' stamp: 'lr 8/30/2008 16:22'!
writeOnLoadOn: aDocument 
	| attributes |
	attributes := WAHtmlAttributes new.
	attributes 
		at: 'type'
		put: 'text/javascript'.
	aDocument 
		openTag: 'script'
		attributes: attributes
		closed: false.
	aDocument nextPutAll: '/*<!![CDATA[*/function onLoad(){'.
	self requestContext loadScripts 
		do: [ :each | aDocument nextPutAll: each seasideString ]
		separatedBy: [ aDocument nextPutAll: ';' ].
	aDocument nextPutAll: '}/*]]>*/'.
	aDocument closeTag: 'script'! !

!WASessionContinuation class methodsFor: 'instance-creation' stamp: 'lr 4/20/2007 20:17'!
root: aComponent
	^ self new initializeWithRoot: aComponent! !

!WASessionContinuation methodsFor: 'processing' stamp: 'lr 4/20/2007 19:34'!
handleRequest: aRequest
	self subclassResponsibility! !

!WASessionContinuation methodsFor: 'initialization' stamp: 'lr 4/20/2007 20:17'!
initializeWithRoot: aComponent
	root := aComponent! !

!WASessionContinuation methodsFor: 'creational' stamp: 'lr 10/28/2007 14:42'!
newRedirectContinuation
	"Answer a new redirect handler."

	^ (self application preferenceAt: #redirectContinuationClass) root: self root! !

!WASessionContinuation methodsFor: 'creational' stamp: 'lr 10/28/2007 14:42'!
newRenderContinuation
	"Answer a new render continuation."

	^ (self application preferenceAt: #renderContinuationClass) root: self root! !

!WASessionContinuation methodsFor: 'creational' stamp: 'lr 10/28/2007 14:42'!
newResponse
	"Answer a new response object and assign the current request stream."

	| stream response |
	stream := self request responseStream.
	response := stream isNil
		ifTrue: [ WAResponse new ]
		ifFalse: [ WAStreamResponse on: stream ].
	^ response doNotCache! !

!WASessionContinuation methodsFor: 'creational' stamp: 'lr 5/10/2007 22:26'!
newStates
	^ WASnapshot new! !

!WASessionContinuation methodsFor: 'accessing' stamp: 'lr 4/9/2008 10:52'!
request
	"Answer the current request object."

	^ self requestContext request! !

!WASessionContinuation methodsFor: 'processing' stamp: 'jf 7/16/2008 10:00'!
respond: aBlock 
	| response context |
	states := self newStates.
	response := self newResponse.
	context := self requestContext.
	context response: response.
	context onRespond: [ :value | self updateStates: states ].
	aBlock value: response.
	context returnResponse: response! !

!WASessionContinuation methodsFor: 'accessing' stamp: 'lr 10/28/2007 14:42'!
root
	"Answer the root component instance. Usually this component is instantiated with the first reuqest."

	^ root! !

!WASessionContinuation methodsFor: 'processing' stamp: 'lr 4/20/2007 19:46'!
run
	self subclassResponsibility! !

!WASessionContinuation methodsFor: 'accessing' stamp: 'lr 8/20/2007 22:59'!
states
	^ states! !

!WASessionContinuation methodsFor: 'updating' stamp: 'lr 4/17/2007 22:50'!
updateRoot: aHtmlRoot
	self session updateRoot: aHtmlRoot.
	self root visiblePresentersDo: [ :each | each updateRoot: aHtmlRoot ]! !

!WASessionContinuation methodsFor: 'updating' stamp: 'lr 10/28/2007 14:42'!
updateStates: aSnapshot
	self session updateStates: aSnapshot.
	self root visiblePresentersDo: [ :each |
		(each isDecoration and: [ each isDelegation ])
			ifTrue: [ each component updateStates: aSnapshot ].
		each updateStates: aSnapshot ]! !

!WASessionContinuation methodsFor: 'updating' stamp: 'lr 4/17/2007 22:50'!
updateUrl: anUrl
	self session updateUrl: anUrl.
	self root visiblePresentersDo: [ :each | each updateUrl: anUrl ]! !

!WASessionContinuation methodsFor: 'accessing' stamp: 'lr 4/17/2007 22:20'!
url
	| url |
	url := self session actionUrlForContinuation: self.
	self updateUrl: url.
	^ url! !

!WASessionContinuation methodsFor: 'evaluating' stamp: 'lr 5/31/2008 16:16'!
value: aRequest
	"Resume processing of aRequest. To ensure valid application state restore all registered states."

	states ifNotNil: [ states restore ].
	self handleRequest: aRequest! !

WAObject subclass: #WASnapshot
	instanceVariableNames: 'objects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WASnapshot commentStamp: 'lr 6/2/2008 09:24' prior: 0!
I represent a memory snapshot of objects. Objects can be registered by sending the message #register:. All object snapshots are updated by sending #snapshot and restored by sending #restore. To have specific snpshot and restore actions for particular classes override #snapshotCopy and #restoreFromSnapshot: in your classes.!

!WASnapshot methodsFor: 'initialize-release' stamp: 'jf 8/30/2008 16:33'!
initialize
	super initialize.
	objects := IdentityDictionary new! !

!WASnapshot methodsFor: 'registry' stamp: 'lr 6/2/2008 09:28'!
register: anObject
	"Register anObject for backtracking."

	anObject ifNotNil: [ objects at: anObject put: anObject snapshotCopy ]! !

!WASnapshot methodsFor: 'actions' stamp: 'lr 5/31/2008 16:28'!
restore
	"Restore all the backtracked states."

	objects keysAndValuesDo: [ :key :value | key restoreFromSnapshot: value ]! !

!WASnapshot methodsFor: 'actions' stamp: 'lr 5/31/2008 16:29'!
snapshot
	"Snapshot all the states that have been registered for backtracking overriding existing snapshots."

	objects associationsDo: [ :assoc | assoc value: assoc key snapshotCopy ]! !

WAMain subclass: #WARenderLoopMain
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WARenderLoopMain commentStamp: 'rew 10/19/2007 10:50' prior: 0!
When a new session on a WAApplication is started WARenderLoopMain initializes the application, that is it:
 
	creates the top level component of the application, 
	informs each component(WAPresenter) of the application that the session started (via WAPresenter>>initialRequest:)
	starts a WARenderLoop to handle the request
!

!WARenderLoopMain methodsFor: 'processing' stamp: 'lr 4/20/2007 15:10'!
call: aComponent
	^ self renderLoopClass new call: aComponent! !

!WARenderLoopMain methodsFor: 'creational' stamp: 'lr 9/14/2008 13:14'!
createRoot
	^ self rootDecorationClasses
		inject: self rootClass new
		into: [ :component :decorationClass |
			component 
				addDecoration: decorationClass new;
				yourself ]! !

!WARenderLoopMain methodsFor: 'accessing' stamp: 'lr 4/20/2007 15:10'!
renderLoopClass
	^ WARenderLoop! !

!WARenderLoopMain methodsFor: 'accessing' stamp: 'lr 9/14/2008 12:56'!
rootClass
	^ self application preferenceAt: #rootClass! !

!WARenderLoopMain methodsFor: 'accessing' stamp: 'lr 9/14/2008 13:03'!
rootDecorationClasses
	^ self application preferenceAt: #rootDecorationClasses! !

!WARenderLoopMain methodsFor: 'processing' stamp: 'lr 9/14/2008 13:15'!
start: aRequest
	| root |
	root := self createRoot.
	root visiblePresentersDo: [ :each | each initialRequest: aRequest ].
	(self renderLoopClass new root: root) run! !

WASystemConfiguration subclass: #WARenderLoopConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WARenderLoopConfiguration commentStamp: 'jf 7/22/2008 12:32' prior: 0!
WARenderLoopConfiguration defines attributes required by the Render Loop classes (see the Seaside-Core-RenderLoop category). Any application using these classes (e.g. WARenderLoopMain) should include this configuration (this is done by default for new applications).

See #addAttributes: for details on provided attributes.!

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'lr 9/14/2008 16:34'!
describeOn: config
	(config list: #redirectContinuationClass)
		options: [self redirectContinuationClasses];
		default: WARedirectContinuation.
	(config list: #renderContinuationClass)
		options: [self renderContinuationClasses];
		default: WARenderContinuation.
	(config list: #rootClass)
		comment: 'The root component of this seaside application.';
		options: [self rootClasses].
	(config collection: #rootDecorationClasses)
		comment: 'The decoration clases that should wrap the root component.';
		options: [self rootDecorationClasses].
	config
		at: #mainClass put: WARenderLoopMain! !

!WARenderLoopConfiguration methodsFor: 'ancestry' stamp: 'jf 7/24/2008 19:10'!
parents
	"We override several attributes of the session configuration."
	^ Array with: WASessionConfiguration instance! !

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'pmm 8/1/2008 12:09'!
redirectContinuationClasses
	^ WARedirectContinuation withAllSubclasses asSortedCollection: [ :a :b | a name <= b name ]! !

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'pmm 8/1/2008 12:09'!
renderContinuationClasses
	^ WARenderContinuation withAllSubclasses asSortedCollection: [ :a :b | a name <= b name ]! !

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'jf 7/29/2008 14:30'!
rootClasses
	^ (WAComponent allSubclasses select: [ :each | each canBeRoot ])
		asSortedCollection: [ :a :b | a name <= b name ]! !

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'lr 9/14/2008 12:58'!
rootDecorationClasses
	^ (WADecoration allSubclasses select: [ :each | each canBeRoot ])
		asSortedCollection: [ :a :b | a name <= b name ]! !
